@@ -19,14 +19,14 @@ class ButtonsModel : public QAbstractListModel
 {
     Q_OBJECT
 public:
-    explicit ButtonsModel(const QVector<DecorationButtonType> &buttons, QObject *parent = nullptr);
+    explicit ButtonsModel(const QList<DecorationButtonType> &buttons, QObject *parent = nullptr);
     explicit ButtonsModel(QObject *parent = nullptr);
     ~ButtonsModel() override;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     QHash<int, QByteArray> roleNames() const override;
 
-    QVector<DecorationButtonType> buttons() const
+    QList<DecorationButtonType> buttons() const
     {
         return m_buttons;
     }
@@ -37,12 +37,12 @@ public:
     Q_INVOKABLE void down(int index);
     Q_INVOKABLE void move(int sourceIndex, int targetIndex);
 
-    void replace(const QVector<DecorationButtonType> &buttons);
+    void replace(const QList<DecorationButtonType> &buttons);
     void add(DecorationButtonType type);
     Q_INVOKABLE void add(int index, int type);
 
 private:
-    QVector<DecorationButtonType> m_buttons;
+    QList<DecorationButtonType> m_buttons;
 };
 
 }
