@@ -14,7 +14,7 @@
 #include "cursor.h"
 #include "kwinscreencast_logging.h"
 #include "main.h"
-#include "opengl/eglnativefence.h"
+#include "opengl/glplatform.h"
 #include "opengl/gltexture.h"
 #include "opengl/glutils.h"
 #include "pipewirecore.h"
@@ -74,11 +74,11 @@ static spa_video_format drmFourCCToSpaVideoFormat(quint32 format)
 void ScreenCastStream::onStreamStateChanged(pw_stream_state old, pw_stream_state state, const char *error_message)
 {
     qCDebug(KWIN_SCREENCAST) << "state changed" << pw_stream_state_as_string(old) << " -> " << pw_stream_state_as_string(state) << error_message;
+    if (m_stopped) {
+        return;
+    }
 
     m_streaming = false;
-    m_pendingBuffer = nullptr;
-    m_pendingNotifier.reset();
-    m_pendingFence.reset();
 
     switch (state) {
     case PW_STREAM_STATE_ERROR:
@@ -97,9 +97,7 @@ void ScreenCastStream::onStreamStateChanged(pw_stream_state old, pw_stream_state
     case PW_STREAM_STATE_CONNECTING:
         break;
     case PW_STREAM_STATE_UNCONNECTED:
-        if (!m_stopped) {
-            Q_EMIT stopStreaming();
-        }
+        stop();
         break;
     }
 }
@@ -120,7 +118,7 @@ void ScreenCastStream::newStreamParams()
     struct spa_pod_frame f;
     spa_pod_builder_push_object(&pod_builder, &f, SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers);
     spa_pod_builder_add(&pod_builder,
-                        SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(16, 2, 16),
+                        SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(3, 2, 4),
                         SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(buffertypes), 0);
     if (!m_dmabufParams) {
         spa_pod_builder_add(&pod_builder,
@@ -155,6 +153,10 @@ void ScreenCastStream::newStreamParams()
 
 void ScreenCastStream::onStreamParamChanged(uint32_t id, const struct spa_pod *format)
 {
+    if (m_stopped) {
+        return;
+    }
+
     if (!format || id != SPA_PARAM_Format) {
         return;
     }
@@ -215,13 +217,13 @@ void ScreenCastStream::onStreamParamChanged(uint32_t id, const struct spa_pod *f
 
 void ScreenCastStream::onStreamAddBuffer(pw_buffer *buffer)
 {
-    struct spa_data *spa_data = buffer->buffer->datas;
-
-    spa_data->mapoffset = 0;
-    spa_data->flags = SPA_DATA_FLAG_READWRITE;
+    if (m_stopped) {
+        return;
+    }
 
     std::shared_ptr<ScreenCastDmaBufTexture> dmabuff;
 
+    struct spa_data *spa_data = buffer->buffer->datas;
     if (spa_data[0].type != SPA_ID_INVALID && spa_data[0].type & (1 << SPA_DATA_DmaBuf)) {
         Q_ASSERT(m_dmabufParams);
         dmabuff = createDmaBufTexture(*m_dmabufParams);
@@ -229,32 +231,38 @@ void ScreenCastStream::onStreamAddBuffer(pw_buffer *buffer)
 
     if (dmabuff) {
         const DmaBufAttributes *dmabufAttribs = dmabuff->buffer()->dmabufAttributes();
-        spa_data->maxsize = dmabufAttribs->pitch[0] * m_resolution.height();
-
         Q_ASSERT(buffer->buffer->n_datas >= uint(dmabufAttribs->planeCount));
         for (int i = 0; i < dmabufAttribs->planeCount; ++i) {
-            buffer->buffer->datas[i].type = SPA_DATA_DmaBuf;
-            buffer->buffer->datas[i].fd = dmabufAttribs->fd[i].get();
-            buffer->buffer->datas[i].data = nullptr;
+            spa_data[i].type = SPA_DATA_DmaBuf;
+            spa_data[i].flags = SPA_DATA_FLAG_READWRITE;
+            spa_data[i].mapoffset = 0;
+            spa_data[i].maxsize = i == 0 ? dmabufAttribs->pitch[i] * dmabufAttribs->height : 0; // TODO: dmabufs don't have a well defined size, it should be zero but some clients check the size to see if the buffer is valid
+            spa_data[i].fd = dmabufAttribs->fd[i].get();
+            spa_data[i].data = nullptr;
+            spa_data[i].chunk->offset = dmabufAttribs->offset[i];
+            spa_data[i].chunk->size = spa_data[i].maxsize;
+            spa_data[i].chunk->stride = dmabufAttribs->pitch[i];
+            spa_data[i].chunk->flags = SPA_CHUNK_FLAG_NONE;
         }
         m_dmabufDataForPwBuffer.insert(buffer, dmabuff);
 #ifdef F_SEAL_SEAL // Disable memfd on systems that don't have it, like BSD < 12
     } else {
-        if (!(spa_data[0].type & (1 << SPA_DATA_MemFd))) {
+        if (!(spa_data->type & (1 << SPA_DATA_MemFd))) {
             qCCritical(KWIN_SCREENCAST) << "memfd: Client doesn't support memfd buffer data type";
             return;
         }
 
         const int bytesPerPixel = m_source->hasAlphaChannel() ? 4 : 3;
         const int stride = SPA_ROUND_UP_N(m_resolution.width() * bytesPerPixel, 4);
-        spa_data->maxsize = stride * m_resolution.height();
         spa_data->type = SPA_DATA_MemFd;
+        spa_data->flags = SPA_DATA_FLAG_READWRITE;
+        spa_data->mapoffset = 0;
+        spa_data->maxsize = stride * m_resolution.height();
         spa_data->fd = memfd_create("kwin-screencast-memfd", MFD_CLOEXEC | MFD_ALLOW_SEALING);
         if (spa_data->fd == -1) {
             qCCritical(KWIN_SCREENCAST) << "memfd: Can't create memfd";
             return;
         }
-        spa_data->mapoffset = 0;
 
         if (ftruncate(spa_data->fd, spa_data->maxsize) < 0) {
             qCCritical(KWIN_SCREENCAST) << "memfd: Can't truncate to" << spa_data->maxsize;
@@ -277,6 +285,11 @@ void ScreenCastStream::onStreamAddBuffer(pw_buffer *buffer)
         } else {
             qCDebug(KWIN_SCREENCAST) << "memfd: created successfully" << spa_data->data << spa_data->maxsize;
         }
+
+        spa_data->chunk->offset = 0;
+        spa_data->chunk->size = spa_data->maxsize;
+        spa_data->chunk->stride = stride;
+        spa_data->chunk->flags = SPA_CHUNK_FLAG_NONE;
 #endif
     }
 
@@ -292,15 +305,15 @@ void ScreenCastStream::onStreamRemoveBuffer(pw_buffer *buffer)
     if (spa_data && spa_data->type == SPA_DATA_MemFd) {
         munmap(spa_data->data, spa_data->maxsize);
         close(spa_data->fd);
-    } else if (spa_data && spa_data->type == SPA_DATA_DmaBuf) {
-        for (int i = 0, c = buffer->buffer->n_datas; i < c; ++i) {
-            close(buffer->buffer->datas[i].fd);
-        }
     }
 }
 
 void ScreenCastStream::onStreamRenegotiateFormat(uint64_t)
 {
+    if (m_stopped) {
+        return;
+    }
+
     m_streaming = false; // pause streaming as we wait for the renegotiation
     char buffer[2048];
     auto params = buildFormats(m_dmabufParams.has_value(), buffer);
@@ -313,10 +326,7 @@ ScreenCastStream::ScreenCastStream(ScreenCastSource *source, std::shared_ptr<Pip
     , m_source(source)
     , m_resolution(source->textureSize())
 {
-    connect(source, &ScreenCastSource::closed, this, [this] {
-        m_streaming = false;
-        Q_EMIT stopStreaming();
-    });
+    connect(source, &ScreenCastSource::closed, this, &ScreenCastStream::stop);
 
     m_pwStreamEvents.version = PW_VERSION_STREAM_EVENTS;
     m_pwStreamEvents.add_buffer = [](void *data, struct pw_buffer *buffer) {
@@ -430,13 +440,13 @@ bool ScreenCastStream::createStream()
     }
 
     if (m_cursor.mode == ScreencastV1Interface::Embedded) {
-        connect(Cursors::self(), &Cursors::currentCursorChanged, this, &ScreenCastStream::invalidateCursor);
-        connect(Cursors::self(), &Cursors::positionChanged, this, [this] {
+        m_cursor.changedConnection = connect(Cursors::self(), &Cursors::currentCursorChanged, this, &ScreenCastStream::invalidateCursor);
+        m_cursor.positionChangedConnection = connect(Cursors::self(), &Cursors::positionChanged, this, [this] {
             recordFrame({});
         });
     } else if (m_cursor.mode == ScreencastV1Interface::Metadata) {
-        connect(Cursors::self(), &Cursors::currentCursorChanged, this, &ScreenCastStream::invalidateCursor);
-        connect(Cursors::self(), &Cursors::positionChanged, this, &ScreenCastStream::recordCursor);
+        m_cursor.changedConnection = connect(Cursors::self(), &Cursors::currentCursorChanged, this, &ScreenCastStream::invalidateCursor);
+        m_cursor.positionChangedConnection = connect(Cursors::self(), &Cursors::positionChanged, this, &ScreenCastStream::recordCursor);
     }
 
     return true;
@@ -444,13 +454,25 @@ bool ScreenCastStream::createStream()
 void ScreenCastStream::coreFailed(const QString &errorMessage)
 {
     m_error = errorMessage;
-    Q_EMIT stopStreaming();
+    stop();
 }
 
 void ScreenCastStream::stop()
 {
+    if (m_stopped) {
+        return;
+    }
+
+    m_streaming = false;
     m_stopped = true;
-    delete this;
+    m_pendingFrame.stop();
+
+    disconnect(m_cursor.changedConnection);
+    m_cursor.changedConnection = {};
+    disconnect(m_cursor.positionChangedConnection);
+    m_cursor.positionChangedConnection = {};
+
+    Q_EMIT stopStreaming();
 }
 
 void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
@@ -476,9 +498,6 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
     }
 
     m_pendingDamages = {};
-    if (m_pendingBuffer) {
-        return;
-    }
 
     if (m_waitForNewBuffers) {
         qCWarning(KWIN_SCREENCAST) << "Waiting for new buffers to be created";
@@ -504,7 +523,6 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
     }
 
     struct pw_buffer *buffer = pw_stream_dequeue_buffer(m_pwStream);
-
     if (!buffer) {
         return;
     }
@@ -512,30 +530,29 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
     struct spa_buffer *spa_buffer = buffer->buffer;
     struct spa_data *spa_data = spa_buffer->datas;
 
-    uint8_t *data = (uint8_t *)spa_data->data;
-    if (!data && spa_buffer->datas->type != SPA_DATA_DmaBuf) {
-        qCWarning(KWIN_SCREENCAST) << "Failed to record frame: invalid buffer data";
-        pw_stream_queue_buffer(m_pwStream, buffer);
-        return;
-    }
-
-    spa_data->chunk->offset = 0;
-    spa_data->chunk->flags = SPA_CHUNK_FLAG_NONE;
     static_cast<OpenGLBackend *>(Compositor::self()->backend())->makeCurrent();
-    if (data || spa_data[0].type == SPA_DATA_MemFd) {
+
+    spa_data->chunk->flags = SPA_CHUNK_FLAG_NONE;
+    if (spa_data[0].type == SPA_DATA_MemFd) {
+        uint8_t *data = static_cast<uint8_t *>(spa_data->data);
+        if (!data) {
+            qCWarning(KWIN_SCREENCAST) << "Failed to record frame: invalid buffer data";
+            spa_data->chunk->flags = SPA_CHUNK_FLAG_CORRUPTED;
+            pw_stream_queue_buffer(m_pwStream, buffer);
+            return;
+        }
+
         const bool hasAlpha = m_source->hasAlphaChannel();
         const int bpp = data && !hasAlpha ? 3 : 4;
         const uint stride = SPA_ROUND_UP_N(size.width() * bpp, 4);
 
         if ((stride * size.height()) > spa_data->maxsize) {
             qCDebug(KWIN_SCREENCAST) << "Failed to record frame: frame is too big";
+            spa_data->chunk->flags = SPA_CHUNK_FLAG_CORRUPTED;
             pw_stream_queue_buffer(m_pwStream, buffer);
             return;
         }
 
-        spa_data->chunk->stride = stride;
-        spa_data->chunk->size = stride * size.height();
-
         m_source->render(spa_data, m_videoFormat.format);
 
         auto cursor = Cursors::self()->currentCursor();
@@ -546,19 +563,16 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
             const PlatformCursorImage cursorImage = kwinApp()->cursorImage();
             painter.drawImage(QRect{position.toPoint(), cursorImage.image().size()}, cursorImage.image());
         }
-    } else {
-        auto &buf = m_dmabufDataForPwBuffer[buffer];
-        Q_ASSERT(buf);
-
-        const DmaBufAttributes *dmabufAttribs = buf->buffer()->dmabufAttributes();
-        Q_ASSERT(buffer->buffer->n_datas >= uint(dmabufAttribs->planeCount));
-        for (int i = 0; i < dmabufAttribs->planeCount; ++i) {
-            buffer->buffer->datas[i].chunk->stride = dmabufAttribs->pitch[i];
-            buffer->buffer->datas[i].chunk->offset = dmabufAttribs->offset[i];
+    } else if (spa_data[0].type == SPA_DATA_DmaBuf) {
+        auto dmabuf = m_dmabufDataForPwBuffer.constFind(buffer);
+        if (dmabuf == m_dmabufDataForPwBuffer.constEnd()) {
+            qCDebug(KWIN_SCREENCAST) << "Failed to record frame: no dmabuf data";
+            spa_data->chunk->flags = SPA_CHUNK_FLAG_CORRUPTED;
+            pw_stream_queue_buffer(m_pwStream, buffer);
+            return;
         }
-        spa_data->chunk->size = spa_data->maxsize;
 
-        m_source->render(buf->framebuffer());
+        m_source->render((*dmabuf)->framebuffer());
 
         auto cursor = Cursors::self()->currentCursor();
         if (m_cursor.mode == ScreencastV1Interface::Embedded && includesCursor(cursor)) {
@@ -572,7 +586,7 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
                 }
             }
             if (m_cursor.texture) {
-                GLFramebuffer::pushFramebuffer(buf->framebuffer());
+                GLFramebuffer::pushFramebuffer((*dmabuf)->framebuffer());
 
                 auto shader = ShaderManager::instance()->pushShader(ShaderTrait::MapTexture);
 
@@ -598,6 +612,18 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
                 m_cursor.lastRect = {};
             }
         }
+
+        // Implicit sync is broken on Nvidia.
+        if (GLPlatform::instance()->isNvidia()) {
+            glFinish();
+        } else {
+            glFlush();
+        }
+    } else {
+        qCWarning(KWIN_SCREENCAST, "Failed to record frame: invalid buffer type: %d", spa_data[0].type);
+        spa_data->chunk->flags = SPA_CHUNK_FLAG_CORRUPTED;
+        pw_stream_queue_buffer(m_pwStream, buffer);
+        return;
     }
 
     if (m_cursor.mode == ScreencastV1Interface::Metadata) {
@@ -607,7 +633,7 @@ void ScreenCastStream::recordFrame(const QRegion &_damagedRegion)
 
     addDamage(spa_buffer, damagedRegion);
     addHeader(spa_buffer);
-    tryEnqueue(buffer);
+    enqueue(buffer);
 }
 
 void ScreenCastStream::addHeader(spa_buffer *spaBuffer)
@@ -660,10 +686,6 @@ void ScreenCastStream::recordCursor()
         return;
     }
 
-    if (m_pendingBuffer) {
-        return;
-    }
-
     const char *error = "";
     auto state = pw_stream_get_state(m_pwStream, &error);
     if (state != PW_STREAM_STATE_STREAMING) {
@@ -677,68 +699,30 @@ void ScreenCastStream::recordCursor()
         return;
     }
 
-    m_pendingBuffer = pw_stream_dequeue_buffer(m_pwStream);
-    if (!m_pendingBuffer) {
+    pw_buffer *pwBuffer = pw_stream_dequeue_buffer(m_pwStream);
+    if (!pwBuffer) {
         return;
     }
 
-    struct spa_buffer *spa_buffer = m_pendingBuffer->buffer;
+    struct spa_buffer *spa_buffer = pwBuffer->buffer;
 
     // in pipewire terms, corrupted means "do not look at the frame contents" and here they're empty.
     spa_buffer->datas[0].chunk->flags = SPA_CHUNK_FLAG_CORRUPTED;
-    spa_buffer->datas[0].chunk->size = 0;
 
     sendCursorData(Cursors::self()->currentCursor(),
                    (spa_meta_cursor *)spa_buffer_find_meta_data(spa_buffer, SPA_META_Cursor, sizeof(spa_meta_cursor)));
     addHeader(spa_buffer);
     addDamage(spa_buffer, {});
-    enqueue();
+    enqueue(pwBuffer);
 }
 
-void ScreenCastStream::tryEnqueue(pw_buffer *buffer)
+void ScreenCastStream::enqueue(pw_buffer *pwBuffer)
 {
-    m_pendingBuffer = buffer;
+    pw_stream_queue_buffer(m_pwStream, pwBuffer);
 
-    // The GPU doesn't necessarily process draw commands as soon as they are issued. Thus,
-    // we need to insert a fence into the command stream and enqueue the pipewire buffer
-    // only after the fence is signaled; otherwise stream consumers will most likely see
-    // a corrupted buffer.
-    if (Compositor::self()->scene()->supportsNativeFence()) {
-        Q_ASSERT_X(eglGetCurrentContext(), "tryEnqueue", "no current context");
-        m_pendingFence = std::make_unique<EGLNativeFence>(kwinApp()->outputBackend()->sceneEglDisplayObject());
-        if (!m_pendingFence->isValid()) {
-            qCWarning(KWIN_SCREENCAST) << "Failed to create a native EGL fence";
-            glFinish();
-            enqueue();
-        } else {
-            m_pendingNotifier = std::make_unique<QSocketNotifier>(m_pendingFence->fileDescriptor().get(), QSocketNotifier::Read);
-            connect(m_pendingNotifier.get(), &QSocketNotifier::activated, this, &ScreenCastStream::enqueue);
-        }
-    } else {
-        // The compositing backend doesn't support native fences. We don't have any other choice
-        // but stall the graphics pipeline. Otherwise stream consumers may see an incomplete buffer.
-        glFinish();
-        enqueue();
-    }
-}
-
-void ScreenCastStream::enqueue()
-{
-    Q_ASSERT_X(m_pendingBuffer, "enqueue", "pending buffer must be valid");
-
-    m_pendingFence.reset();
-    m_pendingNotifier.reset();
-
-    if (!m_streaming) {
-        return;
-    }
-    pw_stream_queue_buffer(m_pwStream, m_pendingBuffer);
-
-    if (m_pendingBuffer->buffer->datas[0].chunk->flags != SPA_CHUNK_FLAG_CORRUPTED) {
+    if (pwBuffer->buffer->datas[0].chunk->flags != SPA_CHUNK_FLAG_CORRUPTED) {
         m_lastSent = QDateTime::currentDateTimeUtc();
     }
-
-    m_pendingBuffer = nullptr;
 }
 
 QList<const spa_pod *> ScreenCastStream::buildFormats(bool fixate, char buffer[2048])
