@@ -16,7 +16,6 @@
 #include <QHash>
 #include <QObject>
 #include <QRegion>
-#include <QSocketNotifier>
 #include <QTimer>
 #include <chrono>
 #include <memory>
@@ -32,7 +31,6 @@ namespace KWin
 
 class Cursor;
 class ScreenCastDmaBufTexture;
-class EGLNativeFence;
 class GLTexture;
 class PipeWireCore;
 class ScreenCastSource;
@@ -96,8 +94,7 @@ private:
     void addHeader(spa_buffer *spaBuffer);
     void addDamage(spa_buffer *spaBuffer, const QRegion &damagedRegion);
     void newStreamParams();
-    void tryEnqueue(pw_buffer *buffer);
-    void enqueue();
+    void enqueue(pw_buffer *buffer);
     spa_pod *buildFormat(struct spa_pod_builder *b, enum spa_video_format format, struct spa_rectangle *resolution,
                          struct spa_fraction *defaultFramerate, struct spa_fraction *minFramerate, struct spa_fraction *maxFramerate,
                          const QList<uint64_t> &modifiers, quint32 modifiersFlags);
@@ -133,13 +130,12 @@ private:
         std::unique_ptr<GLTexture> texture;
         bool visible = false;
         bool invalid = true;
+        QMetaObject::Connection changedConnection = QMetaObject::Connection();
+        QMetaObject::Connection positionChangedConnection = QMetaObject::Connection();
     } m_cursor;
 
     QHash<struct pw_buffer *, std::shared_ptr<ScreenCastDmaBufTexture>> m_dmabufDataForPwBuffer;
 
-    pw_buffer *m_pendingBuffer = nullptr;
-    std::unique_ptr<QSocketNotifier> m_pendingNotifier;
-    std::unique_ptr<EGLNativeFence> m_pendingFence;
     quint64 m_sequential = 0;
     bool m_hasDmaBuf = false;
     bool m_waitForNewBuffers = false;
