@@ -231,7 +231,7 @@ void TouchPointsEffect::drawCircleQPainter(const QColor &color, float cx, float
 void TouchPointsEffect::paintScreenSetupGl(const RenderTarget &renderTarget, const QMatrix4x4 &projectionMatrix)
 {
     GLShader *shader = ShaderManager::instance()->pushShader(ShaderTrait::UniformColor);
-    shader->setUniform(GLShader::ModelViewProjectionMatrix, projectionMatrix);
+    shader->setUniform(GLShader::Mat4Uniform::ModelViewProjectionMatrix, projectionMatrix);
     shader->setColorspaceUniformsFromSRGB(renderTarget.colorDescription());
 
     glLineWidth(m_lineWidth);
