@@ -607,10 +607,6 @@ public:
     bool isOnActiveOutput() const;
     Output *output() const;
     void setOutput(Output *output);
-    QPointF clientPos() const
-    {
-        return QPointF(borderLeft(), borderTop());
-    }; // inside of geometry()
     QSizeF clientSize() const;
     /**
      * Returns a rectangle that the window occupies on the screen, including drop-shadows.
@@ -999,12 +995,14 @@ public:
     void setSuspended(bool suspended);
 
     QRectF fullscreenGeometryRestore() const;
+    void setFullscreenGeometryRestore(const QRectF &geom);
     virtual bool isFullScreenable() const;
     virtual bool isFullScreen() const;
     virtual bool isRequestedFullScreen() const;
     virtual void setFullScreen(bool set);
 
     QRectF geometryRestore() const;
+    void setGeometryRestore(const QRectF &rect);
     virtual bool isMaximizable() const;
     virtual MaximizeMode maximizeMode() const;
     virtual MaximizeMode requestedMaximizeMode() const;
@@ -1435,6 +1433,8 @@ Q_SIGNALS:
     void hiddenByShowDesktopChanged();
     void lockScreenOverlayChanged();
     void readyForPaintingChanged();
+    void maximizeGeometryRestoreChanged();
+    void fullscreenGeometryRestoreChanged();
 
 protected:
     Window();
@@ -1549,8 +1549,6 @@ protected:
     int borderRight() const;
     int borderTop() const;
     int borderBottom() const;
-    void setGeometryRestore(const QRectF &rect);
-    void setFullscreenGeometryRestore(const QRectF &geom);
 
     void blockGeometryUpdates(bool block);
     void blockGeometryUpdates();
