@@ -94,6 +94,7 @@ public:
     QSizeF frameSizeToClientSize(const QSizeF &size) const override;
     QSizeF clientSizeToFrameSize(const QSizeF &size) const override;
     QRectF frameRectToBufferRect(const QRectF &rect) const;
+    QPointF wrapperPos() const;
     QSizeF implicitSize() const;
 
     xcb_visualid_t visual() const;
@@ -220,7 +221,7 @@ public:
     QPointF gravityAdjustment(xcb_gravity_t gravity) const;
     const QPointF calculateGravitation(bool invert) const;
 
-    void NETMoveResize(qreal x_root, qreal y_root, NET::Direction direction);
+    void NETMoveResize(qreal x_root, qreal y_root, NET::Direction direction, xcb_button_t button);
     void NETMoveResizeWindow(int flags, qreal x, qreal y, qreal width, qreal height);
     void GTKShowWindowMenu(qreal x_root, qreal y_root);
     void restackWindow(xcb_window_t above, int detail, NET::RequestSource source, xcb_timestamp_t timestamp,
@@ -430,6 +431,7 @@ private:
     QWindow *findInternalWindow() const;
     void checkOutput();
     void associate();
+    void handleXwaylandScaleChanged();
 
     Xcb::Window m_client;
     Xcb::Window m_wrapper;
