@@ -284,6 +284,7 @@ void XdgSurfaceWindow::destroyWindow()
 {
     handleRoleDestroyed();
     markAsDeleted();
+    stopDelayedInteractiveMoveResize();
     if (isInteractiveMoveResize()) {
         leaveInteractiveMoveResize();
         Q_EMIT interactiveMoveResizeFinished();
@@ -695,11 +696,13 @@ void XdgToplevelWindow::handleRolePrecommit()
 {
     auto configureEvent = static_cast<XdgToplevelConfigure *>(lastAcknowledgedConfigure());
     if (configureEvent && decoration() != configureEvent->decoration.get()) {
-        connect(configureEvent->decoration.get(), &KDecoration2::Decoration::bordersChanged, this, [this]() {
-            if (!isDeleted()) {
-                scheduleConfigure();
-            }
-        });
+        if (configureEvent->decoration) {
+            connect(configureEvent->decoration.get(), &KDecoration2::Decoration::bordersChanged, this, [this]() {
+                if (!isDeleted()) {
+                    scheduleConfigure();
+                }
+            });
+        }
 
         setDecoration(configureEvent->decoration);
         updateShadow();
